@@ -39,10 +39,10 @@
 #include "filters.h"
 #include "stack_alloc.h"
 #include "vq.h"
-#include "speex_bits.h"
+#include <speex/speex_bits.h>
 #include "vbr.h"
 #include "misc.h"
-#include "speex_callbacks.h"
+#include <speex/speex_callbacks.h>
 
 #ifdef SLOW_TRIG
 #include "math_approx.h"
@@ -439,7 +439,7 @@ int nb_encode(void *state, float *in, SpeexBits *bits)
    if (st->submodes[st->submodeID] == NULL)
    {
       for (i=0;i<st->frameSize;i++)
-         st->exc[i]=st->exc2[i]=st->sw[i]=0;
+         st->exc[i]=st->exc2[i]=st->sw[i]=VERY_SMALL;
 
       for (i=0;i<st->lpcSize;i++)
          st->mem_sw[i]=0;
@@ -495,7 +495,7 @@ int nb_encode(void *state, float *in, SpeexBits *bits)
    
    /*Quantize and transmit open-loop excitation gain*/
    {
-      int qe = (int)(floor(3.5*log(ol_gain)));
+      int qe = (int)(floor(.5+3.5*log(ol_gain)));
       if (qe<0)
          qe=0;
       if (qe>31)
@@ -584,15 +584,15 @@ int nb_encode(void *state, float *in, SpeexBits *bits)
 
       /* Compute impulse response of A(z/g1) / ( A(z)*A(z/g2) )*/
       for (i=0;i<st->subframeSize;i++)
-         exc[i]=0;
+         exc[i]=VERY_SMALL;
       exc[0]=1;
       syn_percep_zero(exc, st->interp_qlpc, st->bw_lpc1, st->bw_lpc2, syn_resp, st->subframeSize, st->lpcSize, stack);
 
       /* Reset excitation */
       for (i=0;i<st->subframeSize;i++)
-         exc[i]=0;
+         exc[i]=VERY_SMALL;
       for (i=0;i<st->subframeSize;i++)
-         exc2[i]=0;
+         exc2[i]=VERY_SMALL;
 
       /* Compute zero response of A(z/g1) / ( A(z/g2) * A(z) ) */
       for (i=0;i<st->lpcSize;i++)
@@ -905,7 +905,7 @@ static void nb_decode_lost(DecState *st, float *out, char *stack)
    if (pitch_gain>.95)
       pitch_gain=.95;
 
-   pitch_gain *= fact;
+   pitch_gain = pitch_gain*fact + VERY_SMALL;
 
    /* Shift all buffers by one frame */
    speex_move(st->inBuf, st->inBuf+st->frameSize, (st->bufSize-st->frameSize)*sizeof(float));
@@ -968,13 +968,13 @@ static void nb_decode_lost(DecState *st, float *out, char *stack)
          /*rand();*/
 #else
          /*exc[i]=pitch_gain*exc[i-st->last_pitch] +  fact*st->innov[i+offset];*/
-         exc[i]=pitch_gain*exc[i-st->last_pitch] + 
+         exc[i]=pitch_gain*(exc[i-st->last_pitch]+VERY_SMALL) + 
          fact*sqrt(1-pitch_gain)*speex_rand(innov_gain);
 #endif
       }
       }
       for (i=0;i<st->subframeSize;i++)
-         sp[i]=exc[i];
+         sp[i]=exc[i]+VERY_SMALL;
       
       /* Signal synthesis */
       if (st->lpc_enh_enabled)
@@ -1590,6 +1590,9 @@ int nb_encoder_ctl(void *state, int request, void *ptr)
             st->excBuf[i]=st->swBuf[i]=st->inBuf[i]=st->exc2Buf[i]=0;
       }
       break;
+   case SPEEX_GET_LOOKAHEAD:
+      (*(int*)ptr)=(st->windowSize-st->frameSize);
+      break;
    case SPEEX_GET_PI_GAIN:
       {
          int i;
