@@ -38,11 +38,11 @@
 #include <string.h>
 #include <time.h>
 
-#include "speex.h"
+#include <speex/speex.h>
 #include <ogg/ogg.h>
 #include "wav_io.h"
-#include "speex_header.h"
-#include "speex_stereo.h"
+#include <speex/speex_header.h>
+#include <speex/speex_stereo.h>
 #include "misc.h"
 
 #if defined WIN32 || defined _WIN32
@@ -71,7 +71,7 @@ int oe_write_page(ogg_page *page, FILE *fp)
 #define MAX_FRAME_BYTES 2000
 
 /* Convert input audio bits, endians and channels */
-static int read_samples(FILE *fin,int frame_size, int bits, int channels, int lsb, float * input, char *buff, int *size)
+static int read_samples(FILE *fin,int frame_size, int bits, int channels, int lsb, short * input, char *buff, int *size)
 {   
    unsigned char in[MAX_FRAME_BYTES*2];
    int i;
@@ -80,7 +80,7 @@ static int read_samples(FILE *fin,int frame_size, int bits, int channels, int ls
 
    if (size && *size<=0)
    {
-      return 1;
+      return 0;
    }
    /*Read input audio*/
    if (size)
@@ -99,7 +99,7 @@ static int read_samples(FILE *fin,int frame_size, int bits, int channels, int ls
 
    /*fprintf (stderr, "%d\n", nb_read);*/
    if (nb_read==0)
-      return 1;
+      return 0;
 
    s=(short*)in;
    if(bits==8)
@@ -121,6 +121,7 @@ static int read_samples(FILE *fin,int frame_size, int bits, int channels, int ls
       }
    }
 
+   /* FIXME: This is probably redundent now */
    /* copy to float input buffer */
    for (i=0;i<frame_size*channels;i++)
    {
@@ -133,18 +134,18 @@ static int read_samples(FILE *fin,int frame_size, int bits, int channels, int ls
    }
 
 
-   return 0;
+   return nb_read;
 }
 
 void version()
 {
-   printf ("speexenc (Speex encoder) version " VERSION " (compiled " __DATE__ ")\n");
+   printf ("speexenc (Speex encoder) version " SPEEX_VERSION " (compiled " __DATE__ ")\n");
    printf ("Copyright (C) 2002-2003 Jean-Marc Valin\n");
 }
 
 void version_short()
 {
-   printf ("speexenc version " VERSION "\n");
+   printf ("speexenc version " SPEEX_VERSION "\n");
    printf ("Copyright (C) 2002-2003 Jean-Marc Valin\n");
 }
 
@@ -199,18 +200,21 @@ void usage()
 
 int main(int argc, char **argv)
 {
+   int nb_samples, total_samples=0, nb_encoded;
    int c;
    int option_index = 0;
    char *inFile, *outFile;
    FILE *fin, *fout;
-   float input[MAX_FRAME_SIZE];
+   short input[MAX_FRAME_SIZE];
    int frame_size;
+   int quiet=0;
    int vbr_enabled=0;
    int abr_enabled=0;
    int vad_enabled=0;
    int dtx_enabled=0;
    int nbBytes;
    SpeexMode *mode=NULL;
+   int modeID = -1;
    void *st;
    SpeexBits bits;
    char cbits[MAX_FRAME_BYTES];
@@ -228,6 +232,7 @@ int main(int argc, char **argv)
       {"nframes", required_argument, NULL, 0},
       {"comp", required_argument, NULL, 0},
       {"help", no_argument, NULL, 0},
+      {"quiet", no_argument, NULL, 0},
       {"le", no_argument, NULL, 0},
       {"be", no_argument, NULL, 0},
       {"8bit", no_argument, NULL, 0},
@@ -256,7 +261,7 @@ int main(int argc, char **argv)
    SpeexHeader header;
    int nframes=1;
    int complexity=3;
-   char *vendor_string = "Encoded with Speex " VERSION;
+   char *vendor_string = "Encoded with Speex " SPEEX_VERSION;
    char *comments;
    int comments_length;
    int close_in=0, close_out=0;
@@ -266,7 +271,8 @@ int main(int argc, char **argv)
    char first_bytes[12];
    int wave_input=0;
    int tmp;
-
+   int lookahead = 0;
+   
    comment_init(&comments, &comments_length, vendor_string);
 
    /*Process command-line options*/
@@ -282,13 +288,13 @@ int main(int argc, char **argv)
       case 0:
          if (strcmp(long_options[option_index].name,"narrowband")==0)
          {
-            mode=&speex_nb_mode;
+            modeID = SPEEX_MODEID_NB;
          } else if (strcmp(long_options[option_index].name,"wideband")==0)
          {
-            mode=&speex_wb_mode;
+            modeID = SPEEX_MODEID_WB;
          } else if (strcmp(long_options[option_index].name,"ultra-wideband")==0)
          {
-            mode=&speex_uwb_mode;
+            modeID = SPEEX_MODEID_UWB;
          } else if (strcmp(long_options[option_index].name,"vbr")==0)
          {
             vbr_enabled=1;
@@ -327,6 +333,9 @@ int main(int argc, char **argv)
          {
             usage();
             exit(0);
+         } else if (strcmp(long_options[option_index].name,"quiet")==0)
+         {
+            quiet = 1;
          } else if (strcmp(long_options[option_index].name,"version")==0)
          {
             version();
@@ -355,6 +364,12 @@ int main(int argc, char **argv)
             rate=atoi (optarg);
          } else if (strcmp(long_options[option_index].name,"comment")==0)
          {
+	   if (!strchr(optarg, '='))
+	   {
+	     fprintf (stderr, "Invalid comment: %s\n", optarg);
+	     fprintf (stderr, "Comments must be of the form name=value\n");
+	     exit(1);
+	   }
            comment_add(&comments, &comments_length, NULL, optarg); 
          } else if (strcmp(long_options[option_index].name,"author")==0)
          {
@@ -366,7 +381,7 @@ int main(int argc, char **argv)
 
          break;
       case 'n':
-         mode=&speex_nb_mode;
+         modeID = SPEEX_MODEID_NB;
          break;
       case 'h':
          usage();
@@ -380,10 +395,10 @@ int main(int argc, char **argv)
          print_bitrate=1;
          break;
       case 'w':
-         mode=&speex_wb_mode;
+         modeID = SPEEX_MODEID_WB;
          break;
       case 'u':
-         mode=&speex_uwb_mode;
+         modeID = SPEEX_MODEID_UWB;
          break;
       case '?':
          usage();
@@ -416,11 +431,7 @@ int main(int argc, char **argv)
    }
    else 
    {
-#if defined WIN32 || defined _WIN32
       fin = fopen(inFile, "rb");
-#else
-      fin = fopen(inFile, "r");
-#endif
       if (!fin)
       {
          perror(inFile);
@@ -440,12 +451,12 @@ int main(int argc, char **argv)
       }
    }
 
-   if (!mode && !rate)
+   if (modeID==-1 && !rate)
    {
       /* By default, use narrowband/8 kHz */
-      mode=&speex_nb_mode;
+      modeID = SPEEX_MODEID_NB;
       rate=8000;
-   } else if (mode && rate)
+   } else if (modeID!=-1 && rate)
    {
       if (rate>48000)
       {
@@ -453,19 +464,19 @@ int main(int argc, char **argv)
          exit(1);
       } else if (rate>25000)
       {
-         if (mode!=&speex_uwb_mode)
+         if (modeID != SPEEX_MODEID_UWB)
          {
             fprintf (stderr, "Warning: Trying to encode in %s at %d Hz. I'll do it but I suggest you try ultra-wideband instead\n", mode->modeName , rate);
          }
       } else if (rate>12500)
       {
-         if (mode!=&speex_wb_mode)
+         if (modeID != SPEEX_MODEID_WB)
          {
             fprintf (stderr, "Warning: Trying to encode in %s at %d Hz. I'll do it but I suggest you try wideband instead\n", mode->modeName , rate);
          }
       } else if (rate>=6000)
       {
-         if (mode!=&speex_nb_mode)
+         if (modeID != SPEEX_MODEID_NB)
          {
             fprintf (stderr, "Warning: Trying to encode in %s at %d Hz. I'll do it but I suggest you try narrowband instead\n", mode->modeName , rate);
          }
@@ -473,7 +484,7 @@ int main(int argc, char **argv)
          fprintf (stderr, "Error: sampling rate too low: %d Hz\n", rate);
          exit(1);
       }
-   } else if (!mode)
+   } else if (modeID==-1)
    {
       if (rate>48000)
       {
@@ -481,29 +492,32 @@ int main(int argc, char **argv)
          exit(1);
       } else if (rate>25000)
       {
-         mode=&speex_uwb_mode;
+         modeID = SPEEX_MODEID_UWB;
       } else if (rate>12500)
       {
-         mode=&speex_wb_mode;
+         modeID = SPEEX_MODEID_WB;
       } else if (rate>=6000)
       {
-         mode=&speex_nb_mode;
+         modeID = SPEEX_MODEID_NB;
       } else {
          fprintf (stderr, "Error: Sampling rate too low: %d Hz\n", rate);
          exit(1);
       }
    } else if (!rate)
    {
-      if (mode==&speex_nb_mode)
+      if (modeID == SPEEX_MODEID_NB)
          rate=8000;
-      else if (mode==&speex_wb_mode)
+      else if (modeID == SPEEX_MODEID_WB)
          rate=16000;
-      else if (mode==&speex_uwb_mode)
+      else if (modeID == SPEEX_MODEID_UWB)
          rate=32000;
    }
 
-   if (rate!=8000 && rate!=16000 && rate!=32000)
-      fprintf (stderr, "Warning: Speex is only optimized for 8, 16 and 32 kHz. It will still work at %d Hz but your mileage may vary\n", rate); 
+   if (!quiet)
+      if (rate!=8000 && rate!=16000 && rate!=32000)
+         fprintf (stderr, "Warning: Speex is only optimized for 8, 16 and 32 kHz. It will still work at %d Hz but your mileage may vary\n", rate); 
+
+   mode = speex_lib_get_mode (modeID);
 
    speex_init_header(&header, rate, 1, mode);
    header.frames_per_packet=nframes;
@@ -514,7 +528,8 @@ int main(int argc, char **argv)
       char *st_string="mono";
       if (chan==2)
          st_string="stereo";
-      fprintf (stderr, "Encoding %d Hz audio using %s mode (%s)\n", 
+      if (!quiet)
+         fprintf (stderr, "Encoding %d Hz audio using %s mode (%s)\n", 
                header.rate, mode->modeName, st_string);
    }
    /*fprintf (stderr, "Encoding %d Hz audio at %d bps using %s mode\n", 
@@ -532,11 +547,7 @@ int main(int argc, char **argv)
    }
    else 
    {
-#if defined WIN32 || defined _WIN32
       fout = fopen(outFile, "wb");
-#else
-      fout = fopen(outFile, "w");
-#endif
       if (!fout)
       {
          perror(outFile);
@@ -545,42 +556,6 @@ int main(int argc, char **argv)
       close_out=1;
    }
 
-
-   /*Write header (format will change)*/
-   {
-
-      op.packet = (unsigned char *)speex_header_to_packet(&header, (int*)&(op.bytes));
-      op.b_o_s = 1;
-      op.e_o_s = 0;
-      op.granulepos = 0;
-      op.packetno = 0;
-      ogg_stream_packetin(&os, &op);
-      free(op.packet);
-
-      op.packet = (unsigned char *)comments;
-      op.bytes = comments_length;
-      op.b_o_s = 0;
-      op.e_o_s = 0;
-      op.granulepos = 0;
-      op.packetno = 1;
-      ogg_stream_packetin(&os, &op);
-      
-      while((result = ogg_stream_flush(&os, &og)))
-      {
-         if(!result) break;
-         ret = oe_write_page(&og, fout);
-         if(ret != og.header_len + og.body_len)
-         {
-            fprintf (stderr,"Error: failed writing header to output stream\n");
-            exit(1);
-         }
-         else
-            bytes_written += ret;
-      }
-   }
-
-   free(comments);
-
    speex_encoder_ctl(st, SPEEX_GET_FRAME_SIZE, &frame_size);
    speex_encoder_ctl(st, SPEEX_SET_COMPLEXITY, &complexity);
    speex_encoder_ctl(st, SPEEX_SET_SAMPLING_RATE, &rate);
@@ -622,25 +597,66 @@ int main(int argc, char **argv)
       speex_encoder_ctl(st, SPEEX_SET_ABR, &abr_enabled);
    }
 
+   speex_encoder_ctl(st, SPEEX_GET_LOOKAHEAD, &lookahead);
+   
+   /*Write header*/
+   {
+
+      op.packet = (unsigned char *)speex_header_to_packet(&header, (int*)&(op.bytes));
+      op.b_o_s = 1;
+      op.e_o_s = 0;
+      op.granulepos = 0;
+      op.packetno = 0;
+      ogg_stream_packetin(&os, &op);
+      free(op.packet);
+
+      op.packet = (unsigned char *)comments;
+      op.bytes = comments_length;
+      op.b_o_s = 0;
+      op.e_o_s = 0;
+      op.granulepos = 0;
+      op.packetno = 1;
+      ogg_stream_packetin(&os, &op);
+      
+      while((result = ogg_stream_flush(&os, &og)))
+      {
+         if(!result) break;
+         ret = oe_write_page(&og, fout);
+         if(ret != og.header_len + og.body_len)
+         {
+            fprintf (stderr,"Error: failed writing header to output stream\n");
+            exit(1);
+         }
+         else
+            bytes_written += ret;
+      }
+   }
+
+   free(comments);
+
    speex_bits_init(&bits);
 
    if (!wave_input)
    {
-      if (read_samples(fin,frame_size,fmt,chan,lsb,input, first_bytes, NULL))
-         eos=1;
+      nb_samples = read_samples(fin,frame_size,fmt,chan,lsb,input, first_bytes, NULL);
    } else {
-      if (read_samples(fin,frame_size,fmt,chan,lsb,input, NULL, &size))
-         eos=1;
+      nb_samples = read_samples(fin,frame_size,fmt,chan,lsb,input, NULL, &size);
    }
+   if (nb_samples==0)
+      eos=1;
+   total_samples += nb_samples;
+   nb_encoded = -lookahead;
    /*Main encoding loop (one frame per iteration)*/
-   while (!eos)
+   while (!eos || total_samples>nb_encoded)
    {
       id++;
       /*Encode current frame*/
       if (chan==2)
-         speex_encode_stereo(input, frame_size, &bits);
-      speex_encode(st, input, &bits);
+         speex_encode_stereo_int(input, frame_size, &bits);
+
+      speex_encode_int(st, input, &bits);
       
+      nb_encoded += frame_size;
       if (print_bitrate) {
          int tmp;
          char ch=13;
@@ -648,27 +664,31 @@ int main(int argc, char **argv)
          fputc (ch, stderr);
          cumul_bits += tmp;
          enc_frames += 1;
-         if (vad_enabled || vbr_enabled || abr_enabled)
-            fprintf (stderr, "Bitrate is use: %d bps  (average %d bps)   ", tmp, (int)(cumul_bits/enc_frames));
-         else
-            fprintf (stderr, "Bitrate is use: %d bps     ", tmp);
+         if (!quiet)
+         {
+            if (vad_enabled || vbr_enabled || abr_enabled)
+               fprintf (stderr, "Bitrate is use: %d bps  (average %d bps)   ", tmp, (int)(cumul_bits/enc_frames));
+            else
+               fprintf (stderr, "Bitrate is use: %d bps     ", tmp);
+         }
          
       }
 
       if (wave_input)
       {
-         if (read_samples(fin,frame_size,fmt,chan,lsb,input, NULL, &size))
-         {
-            eos=1;
-            op.e_o_s = 1;
-         }
+         nb_samples = read_samples(fin,frame_size,fmt,chan,lsb,input, NULL, &size);
       } else {
-         if (read_samples(fin,frame_size,fmt,chan,lsb,input, NULL, NULL))
-         {
-            eos=1;
-            op.e_o_s = 1;
-         }
+         nb_samples = read_samples(fin,frame_size,fmt,chan,lsb,input, NULL, NULL);
       }
+      if (nb_samples==0)
+      {
+         eos=1;
+      }
+      if (eos && total_samples<=nb_encoded)
+         op.e_o_s = 1;
+      else
+         op.e_o_s = 0;
+      total_samples += nb_samples;
 
       if ((id+1)%nframes!=0)
          continue;
@@ -678,11 +698,15 @@ int main(int argc, char **argv)
       op.packet = (unsigned char *)cbits;
       op.bytes = nbBytes;
       op.b_o_s = 0;
-      if (eos)
+      /*Is this redundent?*/
+      if (eos && total_samples<=nb_encoded)
          op.e_o_s = 1;
       else
          op.e_o_s = 0;
-      op.granulepos = (id+nframes)*frame_size;
+      op.granulepos = (id+1)*frame_size-lookahead;
+      if (op.granulepos>total_samples)
+         op.granulepos = total_samples;
+      /*printf ("granulepos: %d %d %d %d %d %d\n", (int)op.granulepos, id, nframes, lookahead, 5, 6);*/
       op.packetno = 2+id/nframes;
       ogg_stream_packetin(&os, &op);
 
@@ -711,7 +735,10 @@ int main(int argc, char **argv)
       op.bytes = nbBytes;
       op.b_o_s = 0;
       op.e_o_s = 1;
-      op.granulepos = (id+nframes)*frame_size;
+      op.granulepos = (id+1)*frame_size-lookahead;
+      if (op.granulepos>total_samples)
+         op.granulepos = total_samples;
+
       op.packetno = 2+id/nframes;
       ogg_stream_packetin(&os, &op);
    }
@@ -727,7 +754,6 @@ int main(int argc, char **argv)
       else
          bytes_written += ret;
    }
-   
 
    speex_encoder_destroy(st);
    speex_bits_destroy(&bits);
